#pragma once

#include <fstream>
#include <vector>

#include "DataStructures.h"

class MeshLoader {
private:
    int getLastNodesId();

protected:
    std::vector<Node> nodes;
    std::vector<FiniteElement> finite_elements;
    std::vector<BoundaryFiniteElement> boundary_finite_elements;

public:
    virtual void LoadMesh(const std::string&) = 0;
    virtual ~MeshLoader() = default;

    void PrintData();
    void TransformElementsToQuadr();

    Node GetNodeById(int);

    std::vector<Node>& Nodes();
    std::vector<Node> VertexNodesBySurfaceId(int);
    std::vector<FiniteElement>& FiniteElements();
    std::vector<FiniteElement> FiniteElementsByThreeId(int, int, int);
    std::vector<FiniteElement> FiniteElementsByRib(int, int);
    std::vector<FiniteElement> FiniteElementsByMaterialId(int);
    std::vector<BoundaryFiniteElement>& BoundaryFiniteElements();
    std::vector<BoundaryFiniteElement> BoundaryFiniteElementsById(int);
    std::vector<std::vector<Node>> NodesNeighbors();
    double GetAngle(const FiniteElement& elem);
    void SortByAngle(const std::string& filepath);
};
